searchNodes=[{"doc":"A module for sending and receiving messages to/from an OSC server. Starting a client will create a UDP socket on an arbitrary (system-assigned) port and then wait for messages to be sent or received. No initial negotiation is performed. The client will act as a GenStage producer. To receive responses to your requests, you should create a GenStage consumer (or consumer-producer) and subscribe it to the PID returned by start_link/1 . Each event will be a decoded OSC.Message structure. Due to the stateless nature of the OSC protocol, it is up to the user of this library to ensure there is actually an OSC server at the target IP and port. Failure to do so will not cause any errors on startup, nor prevent sending messages, but will simply result in no actions being performed and no replies being received.","ref":"ExOSC.Client.html","title":"ExOSC.Client","type":"module"},{"doc":"Encodes and sends an OSC.Message to the target.","ref":"ExOSC.Client.html#send_message/2","title":"ExOSC.Client.send_message/2","type":"function"},{"doc":"Starts a client that will send and receive OSC messages to/from a target IP and port. Options :ip (required) - target IP in tuple form :port (required) - target UDP port This function also accepts all the options accepted by GenServer.start_link/3 . Return values Same as GenServer.start_link/3 .","ref":"ExOSC.Client.html#start_link/1","title":"ExOSC.Client.start_link/1","type":"function"},{"doc":"Option values used by start_link/1","ref":"ExOSC.Client.html#t:option/0","title":"ExOSC.Client.option/0","type":"type"},{"doc":"Options used by start_link/1","ref":"ExOSC.Client.html#t:options/0","title":"ExOSC.Client.options/0","type":"type"},{"doc":"A structure representing an OSC message. The same structure is used in both directions, serving as both requests and replies. It consists of a request path (encoded as an OSC.Types.String ), a type tag string (same), and an encoded list of arguments that can be decoded using the type tag string as a reference. For querying device parameters, a typical pattern is that the client will send a message with a given path and empty args , and the server will respond via a message with the same path and the requested data as the args list.","ref":"OSC.Message.html","title":"OSC.Message","type":"module"},{"doc":"Create a message with a given path and (optional) arguments. This is the preferred means of creating OSC.Message structures — in addition to some basic checks on path and args , this will also call OSC.Types.validate_args/1 to ensure that all the arguments can be mapped to OSC types.","ref":"OSC.Message.html#construct/2","title":"OSC.Message.construct/2","type":"function"},{"doc":"Parse a raw binary into an OSC.Message structure. The path and type tag string are decoded using OSC.Types.String.decode/1 , and then the arguments are decoded via OSC.Types.decode_args/2 using the type tag string as a reference. Returns the resulting OSC.Message structure. Raises if there is any unconsumed data after the message ends.","ref":"OSC.Message.html#parse/1","title":"OSC.Message.parse/1","type":"function"},{"doc":"Convert an OSC.Message structure to encoded network format. The arguments will be encoded using OSC.Types.encode_args/1 , and then the message path , type tag string, and encoded arguments will be concatenated to form the packet. Returns the encoded packet as a binary, ready to send via UDP.","ref":"OSC.Message.html#to_packet/1","title":"OSC.Message.to_packet/1","type":"function"},{"doc":"The OSC.Message structure.","ref":"OSC.Message.html#t:t/0","title":"OSC.Message.t/0","type":"type"},{"doc":"Encoding and decoding of the possible args types in an OSC.Message . Encoded messages contain a &quot;type tag string&quot; (encoded as an OSC &quot;string&quot; type), starting with a comma, where each subsequent letter indicates the type of the respective argument. For example, &quot;,sif&quot; indicates a message whose three arguments are an OSC string, integer, and float, respectively. Thus, to decode arguments, we need to know the type tag string; and when encoding arguments, we must also produce a type tag string.","ref":"OSC.Types.html","title":"OSC.Types","type":"module"},{"doc":"Decodes OSC arguments, given a decoded OSC type tag string. The type tag string will need to be decoded first using OSC.Types.String.decode/1 . This function will then determine which type decoder to use for each argument, based on the respective characters in the tag string. Returns {args, rest} where args is the decoded args and rest is any data that was not consumed by the argument decoders. Example iex&gt; OSC.Types . decode_args ( &quot;,iiis&quot; , &lt;&lt; ...&gt; 0 , 0 , 0 , 3 , ...&gt; 0 , 0 , 0 , 2 , ...&gt; 0 , 0 , 0 , 1 , ...&gt; &quot;go&quot; , 0 , 0 , ...&gt; &quot;stop&quot; ...&gt; &gt;&gt; ) { [ 3 , 2 , 1 , &quot;go&quot; ] , &quot;stop&quot; }","ref":"OSC.Types.html#decode_args/2","title":"OSC.Types.decode_args/2","type":"function"},{"doc":"Encode arguments into an OSC type tag string and encoded binaries. Returns {tags, encoded} where tags is the tag string, and encoded is a list of each encoded argument. Note that the tag string will itself need to be encoded before sending, using OSC.Types.String.encode/1 . The encoded tag string can then be concatenated with the remaining binaries to form the argument portion of an OSC.Message . Example iex&gt; OSC.Types . encode_args ( [ 1.0 , 2 , [ 3 ] , &quot;4&quot; ] ) { &quot;,fibs&quot; , [ &lt;&lt; 63 , 128 , 0 , 0 &gt;&gt; , # float: 1.0 &lt;&lt; 0 , 0 , 0 , 2 &gt;&gt; , # int: 2 &lt;&lt; 0 , 0 , 0 , 1 , 3 , 0 , 0 , 0 &gt;&gt; , # blob: [3] &lt;&lt; 52 , 0 , 0 , 0 &gt;&gt; # string: &quot;4&quot; ] }","ref":"OSC.Types.html#encode_args/1","title":"OSC.Types.encode_args/1","type":"function"},{"doc":"Ensure that all arguments can be mapped to OSC types. Raises ArgumentError if an argument is of a type that cannot be encoded. Note that this only serves as a basic sanity check on argument types, and does not actually ensure that arguments can be encoded. For example, it does not check the range of numeric types, nor does it check the contents of blobs. Examples iex&gt; OSC.Types . validate_args ( [ 1 , 2.0 , [ 3 , 4 , 5 ] , &quot;str&quot; ] ) :ok iex&gt; OSC.Types . validate_args ( [ 1 , :atom , 12 ] ) ** (ArgumentError) Unknown OSC type: :atom","ref":"OSC.Types.html#validate_args/1","title":"OSC.Types.validate_args/1","type":"function"},{"doc":"List of OSC message arguments","ref":"OSC.Types.html#t:args/0","title":"OSC.Types.args/0","type":"type"},{"doc":"Values that can be encoded into OSC types","ref":"OSC.Types.html#t:t/0","title":"OSC.Types.t/0","type":"type"},{"doc":"Encoding and decoding of the OSC blob type. Blobs are of a list of arbitrary bytes of a given length. Their encoded form consists of an OSC integer indicating the length, followed by the bytes in raw binary format, then padded with null characters ( &quot;\\0&quot; ) until the result is 32-bit aligned (like all OSC datatypes). As such, this is a composite type that also uses the encoding and decoding functions from OSC.Types.Integer .","ref":"OSC.Types.Blob.html","title":"OSC.Types.Blob","type":"module"},{"doc":"Decodes an OSC blob to a list of bytes. The blob must start with an OSC integer (see Types.Integer.decode/1 ) that indicates the number of bytes in the blob. After reading the blob contents, some additional bytes may be consumed (but discarded) as needed to reach the next 32-bit boundary. Returns {blob, rest} where blob is a list of bytes and rest is a binary containing any data not consumed by the decoder. Examples iex&gt; &lt;&lt; 0 , 0 , 0 , 5 , 1 , 2 , 3 , 4 , 5 , 0 , 0 , 0 , 123 &gt;&gt; |&gt; OSC.Types.Blob . decode ( ) { [ 1 , 2 , 3 , 4 , 5 ] , &lt;&lt; 123 &gt;&gt; } iex&gt; &lt;&lt; 0 , 0 , 0 , 7 , &quot;goodbye world&quot; &gt;&gt; |&gt; OSC.Types.Blob . decode ( ) { &#39;goodbye&#39; , &quot;world&quot; }","ref":"OSC.Types.Blob.html#decode/1","title":"OSC.Types.Blob.decode/1","type":"function"},{"doc":"Encodes a list of bytes to an OSC blob type. The encoded data consists of the number of bytes encoded as an OSC integer (see Types.Integer.encode/1 ), followed by the bytes in raw binary format, followed by zero or more null characters until 32-bit aligned. Examples iex&gt; [ 1 , 2 , 3 , 4 ] |&gt; OSC.Types.Blob . encode ( ) &lt;&lt; 0 , 0 , 0 , 4 , 1 , 2 , 3 , 4 &gt;&gt; iex&gt; [ 1 , 2 , 3 , 4 , 5 ] |&gt; OSC.Types.Blob . encode ( ) &lt;&lt; 0 , 0 , 0 , 5 , 1 , 2 , 3 , 4 , 5 , 0 , 0 , 0 &gt;&gt; iex&gt; &#39;hello world&#39; |&gt; OSC.Types.Blob . encode ( ) &lt;&lt; 0 , 0 , 0 , 11 , 104 , 101 , 108 , 108 , 111 , 32 , 119 , 111 , 114 , 108 , 100 , 0 &gt;&gt;","ref":"OSC.Types.Blob.html#encode/1","title":"OSC.Types.Blob.encode/1","type":"function"},{"doc":"Returns ?b , the type tag for the OSC blob type iex&gt; &lt;&lt; OSC.Types.Blob . type_tag ( ) &gt;&gt; &quot;b&quot;","ref":"OSC.Types.Blob.html#type_tag/0","title":"OSC.Types.Blob.type_tag/0","type":"function"},{"doc":"An OSC blob represented as a list of bytes","ref":"OSC.Types.Blob.html#t:t/0","title":"OSC.Types.Blob.t/0","type":"type"},{"doc":"Encoding and decoding of the OSC float type. OSC floats are encoded in 32-bit IEEE 754 binary format (a.k.a single , or more recently, binary32 ). This makes converting them very easy in Elixir, since we can just use &lt;&lt;x::float-size(32)&gt;&gt; binary notation for both encoding and decoding. Precision and rounding As is always the case when working with floating point numbers, you should expect precision errors to appear rather quickly when using this datatype — especially since 32-bit floats are not especially precise, only reliable to about 7 digits worth. For example, you will almost always get different results if you encode and then immediately decode a float: iex&gt; 0.333 |&gt; OSC.Types.Float . encode ( ) |&gt; OSC.Types.Float . decode ( ) { 0.3330000042915344 , &quot;&quot; } This is also before you get into any precision errors introduced by the OSC-controlled hardware itself. For example, your mixing hardware might store a fader volume as an integer, performing float-to-int conversions on &quot;set&quot; operations and int-to-float conversions on &quot;get&quot; operations. As such, you should always expect a lot of rounding when using the OSC float type, as well as a lot of extraneous digits (that you might want to consider discarding before displaying to the end user).","ref":"OSC.Types.Float.html","title":"OSC.Types.Float","type":"module"},{"doc":"Decodes an OSC float to an Elixir float. Returns {float, rest} where float is a float decoded from the first four bytes of the input, and rest is a binary containing the remaining data. Examples iex&gt; &lt;&lt; 63 , 128 , 0 , 0 &gt;&gt; |&gt; OSC.Types.Float . decode ( ) { 1.0 , &quot;&quot; } iex&gt; &lt;&lt; 68 , 249 , 184 , 0 , 234 &gt;&gt; |&gt; OSC.Types.Float . decode ( ) { 1997.75 , &lt;&lt; 234 &gt;&gt; }","ref":"OSC.Types.Float.html#decode/1","title":"OSC.Types.Float.decode/1","type":"function"},{"doc":"Encodes an Elixir float to an OSC float type. Returns a 32-bit binary-encoded float. Examples iex&gt; OSC.Types.Float . encode ( 0.3333 ) &lt;&lt; 62 , 170 , 166 , 76 &gt;&gt;","ref":"OSC.Types.Float.html#encode/1","title":"OSC.Types.Float.encode/1","type":"function"},{"doc":"Returns ?f , the type tag for the OSC float type iex&gt; &lt;&lt; OSC.Types.Float . type_tag ( ) &gt;&gt; &quot;f&quot;","ref":"OSC.Types.Float.html#type_tag/0","title":"OSC.Types.Float.type_tag/0","type":"function"},{"doc":"An OSC float represented by an Elixir float","ref":"OSC.Types.Float.html#t:t/0","title":"OSC.Types.Float.t/0","type":"type"},{"doc":"Encoding and decoding of the OSC integer type. OSC integers are encoded in 32-bit, signed, big-endian binary format. As such, the lowest possible integer is -2³¹ ( min/0 = -2,147,483,648) and the highest is one less than 2³¹ ( max/0 = 2,147,483,647).","ref":"OSC.Types.Integer.html","title":"OSC.Types.Integer","type":"module"},{"doc":"Decodes an OSC integer to an Elixir integer. Returns {int, rest} where int is an integer decoded from the first four bytes of the input, and rest is a binary containing the remaining data. Examples iex&gt; &lt;&lt; 0 , 0 , 0 , 3 &gt;&gt; |&gt; OSC.Types.Integer . decode ( ) { 3 , &quot;&quot; } iex&gt; &lt;&lt; 0 , 1 , 2 , 3 , 4 , 5 &gt;&gt; |&gt; OSC.Types.Integer . decode ( ) { 66051 , &lt;&lt; 4 , 5 &gt;&gt; }","ref":"OSC.Types.Integer.html#decode/1","title":"OSC.Types.Integer.decode/1","type":"function"},{"doc":"Encodes an Elixir integer to an OSC integer type. Returns a 32-bit big-endian-encoded integer. Will raise an error if the integer is lower than min/0 or higher than max/0 . Examples iex&gt; OSC.Types.Integer . encode ( 123 ) &lt;&lt; 0 , 0 , 0 , 123 &gt;&gt; iex&gt; OSC.Types.Integer . encode ( - 987 ) &lt;&lt; 255 , 255 , 252 , 37 &gt;&gt;","ref":"OSC.Types.Integer.html#encode/1","title":"OSC.Types.Integer.encode/1","type":"function"},{"doc":"Returns the largest possible OSC integer ( 2**31 - 1 ). iex&gt; OSC.Types.Integer . max ( ) 2_147_483_647","ref":"OSC.Types.Integer.html#max/0","title":"OSC.Types.Integer.max/0","type":"function"},{"doc":"Returns the smallest possible OSC integer ( 0 - 2**31 ). iex&gt; OSC.Types.Integer . min ( ) - 2_147_483_648","ref":"OSC.Types.Integer.html#min/0","title":"OSC.Types.Integer.min/0","type":"function"},{"doc":"Returns ?i , the type tag for the OSC integer type iex&gt; &lt;&lt; OSC.Types.Integer . type_tag ( ) &gt;&gt; &quot;i&quot;","ref":"OSC.Types.Integer.html#type_tag/0","title":"OSC.Types.Integer.type_tag/0","type":"function"},{"doc":"An OSC integer, represented by a signed 32-bit Elixir integer","ref":"OSC.Types.Integer.html#t:t/0","title":"OSC.Types.Integer.t/0","type":"type"},{"doc":"Encoding and decoding of the OSC string type. There's minimal encoding and decoding required here, since the only requirement for an OSC string is that it end with a null byte ( &quot;\\0&quot; ) and that it be 32-bit aligned (like all OSC types). As such, encoding just involves adding null bytes, and decoding just involves finding the last block and removing the trailing nulls.","ref":"OSC.Types.String.html","title":"OSC.Types.String","type":"module"},{"doc":<<68, 101, 99, 111, 100, 101, 115, 32, 97, 110, 32, 79, 83, 67, 32, 115, 116, 114, 105, 110, 103, 32, 116, 111, 32, 97, 110, 32, 69, 108, 105, 120, 105, 114, 32, 115, 116, 114, 105, 110, 103, 46, 32, 87, 105, 108, 108, 32, 115, 101, ...>>,"ref":"OSC.Types.String.html#decode/1","title":"OSC.Types.String.decode/1","type":"function"},{"doc":"Encodes an Elixir string to an OSC string type. Returns the encoded string, which will be the original string with 1 to 4 null bytes ( &quot;\\0&quot; ) added as needed (to 32-bit align it). The input string cannot itself contain any null bytes. Examples iex&gt; &quot;hello world&quot; |&gt; OSC.Types.String . encode ( ) &lt;&lt; &quot;hello world&quot; , 0 &gt;&gt; iex&gt; &quot;PadMe&quot; |&gt; OSC.Types.String . encode ( ) &lt;&lt; &quot;PadMe&quot; , 0 , 0 , 0 &gt;&gt; iex&gt; &quot;multiple of four&quot; |&gt; OSC.Types.String . encode ( ) &lt;&lt; &quot;multiple of four&quot; , 0 , 0 , 0 , 0 &gt;&gt;","ref":"OSC.Types.String.html#encode/1","title":"OSC.Types.String.encode/1","type":"function"},{"doc":"Returns ?s , the type tag for the OSC integer type iex&gt; &lt;&lt; OSC.Types.String . type_tag ( ) &gt;&gt; &quot;s&quot;","ref":"OSC.Types.String.html#type_tag/0","title":"OSC.Types.String.type_tag/0","type":"function"},{"doc":"An OSC string, represented by an Elixir binary string","ref":"OSC.Types.String.html#t:t/0","title":"OSC.Types.String.t/0","type":"type"},{"doc":"ExOSC is a library for sending and receiving messages to/from audio hardware that supports the OpenSoundControl 1.0 protocol. The exact capabilities of this library will depend on what hardware it's talking to. For example, when talking to an audio mixer, you'll likely be able to control the volume levels of the various faders, what audio gets routed to what outputs, etc. x32_remote is an example of a library that uses ExOSC to talk to Behringer X32/M32 digital mixing consoles.","ref":"readme.html","title":"ExOSC","type":"extras"},{"doc":"OSC is a stateless protocol that typically operates over UDP. Messages have an address pattern string (or path , for short) that typically points to a resource or a command, and an arguments list comprised of zero or more supported OSC datatypes. The OSC protocol is the same in both directions, i.e. simply passing messages back and forth. While there is no explicit request-reply mechanism — nor any built-in acknowledgement of requests — a typical pattern is for the local agent to issue a request with a given path , and the remote agent to respond using a message with an identical path . A partial list of devices that support OSC can be found on the OSC Wikipedia page .","ref":"readme.html#what-is-osc","title":"ExOSC - What is OSC?","type":"extras"},{"doc":"ExOSC requires Elixir v1.14. To use it, add :ex_osc to your list of dependencies in mix.exs : def deps do [ { :ex_osc , &quot;~&gt; 0.1.0&quot; } ] end","ref":"readme.html#installation","title":"ExOSC - Installation","type":"extras"},{"doc":"Here's an example of using ExOSC to request some basic info from a Behringer X32 digital rackmount mixer: defmodule MyConsumer do use GenStage def init ( :ok ) do { :consumer , nil } end def handle_events ( events , { _ , _ } , state ) do events |&gt; Enum . each ( &amp; IO . inspect / 1 ) { :noreply , [ ] , state } end end { :ok , client } = ExOSC.Client . start_link ( ip : { 192 , 168 , 1 , 123 } , port : 10023 ) { :ok , consumer } = GenStage . start_link ( MyConsumer , :ok , [ ] ) { :ok , _ } = GenStage . sync_subscribe ( consumer , to : client ) ExOSC.Client . send_message ( client , % OSC.Message { path : &quot;/info&quot; } ) ExOSC.Client . send_message ( client , % OSC.Message { path : &quot;/ch/01/mix/fader&quot; } ) ExOSC.Client . send_message ( client , % OSC.Message { path : &quot;/ch/01/mix/pan&quot; } ) Process . sleep ( 100 ) # give it some time to reply Output: % OSC.Message { path : &quot;/info&quot; , args : [ &quot;V2.07&quot; , &quot;osc-server&quot; , &quot;X32RACK&quot; , &quot;4.06-8&quot; ] } % OSC.Message { path : &quot;/ch/01/mix/fader&quot; , args : [ 1.0 ] } % OSC.Message { path : &quot;/ch/01/mix/pan&quot; , args : [ 0.5 ] } Of course, for this particular hardware, you should probably use x32_remote instead.","ref":"readme.html#usage","title":"ExOSC - Usage","type":"extras"},{"doc":"Full documentation can be found at https://hexdocs.pm/ex_osc .","ref":"readme.html#documentation","title":"ExOSC - Documentation","type":"extras"},{"doc":"Copyright © 2023, Adrian Irving-Beer. ExOSC is released under the MIT license and is provided with no warranty . Be careful with what commands you issue to your (usually expensive) audio hardware.","ref":"readme.html#legal-stuff","title":"ExOSC - Legal stuff","type":"extras"}]